package com.abelski.module;

import java.util.*;
import java.io.IOException;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;

public class LoginModuleImplementation implements LoginModule 
{
	private Subject subject;
	private CallbackHandler callbackHandler;
	private Map sharedStateMap;
	private Map options;
	private boolean debug = true;
	private char[] password = null;
	private String username = null;
	private boolean succeeded, commitSucceeded;

    private PrincipleImplementation userPrincipal;

    public void initialize(Subject subject, CallbackHandler callbackHandler,
			Map sharedStateMap, Map options) 
	{
		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.sharedStateMap = sharedStateMap;
		this.options = options;
		debug = "true".equalsIgnoreCase((String)options.get("debug"));
    }

    public boolean login() throws LoginException 
	{
		if(debug) System.out.println("DEBUG within login()");
		if (callbackHandler == null)
			throw new LoginException("error... no CallbackHandler available...");
		Callback[] callbacks = new Callback[2];
		callbacks[0] = new NameCallback("user name: ");
		callbacks[1] = new PasswordCallback("password: ", false);
		try 
		{
			callbackHandler.handle(callbacks);
			username = ((NameCallback)callbacks[0]).getName();
			char[] tmpPassword = ((PasswordCallback)callbacks[1]).getPassword();
			if (tmpPassword == null) 
				tmpPassword = new char[0];
	        password = new char[tmpPassword.length];
			System.arraycopy(tmpPassword, 0,password, 0, tmpPassword.length);
			((PasswordCallback)callbacks[1]).clearPassword();
		} 
		catch (java.io.IOException ioe) 
		{
			throw new LoginException(ioe.toString());
		} 
		catch (UnsupportedCallbackException uce) 
		{
			throw new LoginException("error... ");
		}

		if (username.equals("user") && new String(password).equals("password"))
		{
			succeeded = true;
			return true;
		} 
		else 
		{
			System.out.println("authentication failed...");
			succeeded = false;
			if (!username.equals("user")) 
			{
				throw new FailedLoginException("User Name Incorrect");
			} 
			else 
			{
				throw new FailedLoginException("Password Incorrect");
			}
		}
    }

    public boolean commit() throws LoginException 
	{
		if(debug) System.out.println("DEBUG within commit()");	
		if (succeeded == false) 
		{
			if(debug) System.out.println("DEBUG within commit()... succeeded is false...");		
			return false;
		} 
		else 
		{
			if(debug) System.out.println("DEBUG within commit()... succeeded is true...");				
			userPrincipal = new PrincipleImplementation(username);
			if(debug) System.out.println("DEBUG within commit()... succeeded is true... userPrincipal was instantiated...");					
			if (!subject.getPrincipals().contains(userPrincipal))
			{
				if(debug) System.out.println("DEBUG within commit()... succeeded is true... userPrincipal isn't contained within principals... ");		
				subject.getPrincipals().add(userPrincipal);
				if(debug) System.out.println("DEBUG within commit()... succeeded is true... userPrincipal was added to principals...");		
			}
			username = null;
			password = null;
			commitSucceeded = true;
			return true;
		}
    }

    public boolean abort() throws LoginException 
	{
		if(debug) System.out.println("DEBUG within abort()... succeeded="+succeeded+" commitSucceeded="+commitSucceeded);
		if (succeeded == false) 
		{
			return false;
		} 
		else if (succeeded == true && commitSucceeded == false) 
		{
			succeeded = false;
			username = null;
			if (password != null) 
			password = null;
	        userPrincipal = null;
		} 
		else 
		{
			logout();
		}
		return true;
    }

    public boolean logout() throws LoginException 
	{
		if(debug) System.out.println("DEBUG within logout()");	
		subject.getPrincipals().remove(userPrincipal);
		succeeded = false;
		succeeded = commitSucceeded;
		username = null;
		if (password != null) 
		{
			for(int index=0; index<password.length; index++)
			{
				password[index] = 0;
			}
			password = null;
		}
		userPrincipal = null;
		return true;
    }
}
